/*
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This code is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this code; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.  */

/* Copyright (C) 2002 Ian Zimmerman <itz@speakeasy.org> */

static const char rcsid[] = "$Id: gim_connection.cc,v 1.3 2002/08/18 15:19:18 itz Exp $ ";

#ifdef __cplusplus

#include "gim_connection.hh"
#include "gim_init_counter.hh"
#include <sys/time.h>
#include <sys/select.h>

static gim_init_counter cnt;

gim_exception::gim_exception(int what)
  : what (what)
{

}

const char* gim_exception::to_string()
{
  return gim_error_string(what);
}

gim_connection::gim_connection(struct gim_client_masks* p_masks, int vt, const char* device)
  : masks(*p_masks), vt(vt), device(device), fd(-1)
{

}

gim_connection::~gim_connection()
{
  try {
    close();
  }
  catch (struct gim_exception e) {
  }
}

void gim_connection::open() throw (struct gim_exception)
{
  int res = gim_client_connect(&masks, vt, device);
  if (res < 0) {
    throw gim_exception(res);
  } /*if*/
  fd = res;
}

void gim_connection::close() throw (struct gim_exception)
{
  if (fd != -1) {
    int res = gim_client_close(fd);
    if (res != GIM_CLOSE_SUCCESS) {
      throw gim_exception(res);
    } /*if*/
    fd = -1;
  } /*if*/
}

void gim_connection::set_masks(const struct gim_client_masks* p_masks) throw (struct gim_exception)
{
  masks = *p_masks;
  if (fd != -1) {
    int res = gim_client_set_masks(fd, &masks);
    if (res != GIM_SET_MASKS_SUCCESS) {
      throw gim_exception(res);
    } /*if*/
  } /*if*/
}

void gim_connection::get_masks(struct gim_client_masks* p_masks)
{
  *p_masks = masks;
}

void gim_connection::draw_pointer(int x, int y) throw (struct gim_exception)
{
  if (fd != -1) {
    int res = gim_client_draw_pointer(fd, x, y);
    if (res != GIM_DRAW_POINTER_SUCCESS) {
      throw gim_exception(res);
    } /*if*/
  } /*if*/
}

void gim_connection::stop() throw (struct gim_exception)
{
  if (fd != -1) {
    int res = gim_client_stop(fd);
    if (res != GIM_STOP_SUCCESS) {
      throw gim_exception(res);
    } /*if*/
  } /*if*/
}

void gim_connection::start() throw (struct gim_exception)
{
  if (fd != -1) {
    int res = gim_client_start(fd);
    if (res != GIM_START_SUCCESS) {
      throw gim_exception(res);
    } /*if*/
  } /*if*/
}

int gim_connection::devinfo_len() throw (struct gim_exception)
{
  if (fd != -1) {
    int res = gim_client_devinfo(fd, 0, 0);
    if (res < 0) {
      throw gim_exception(res);
    } /*if*/
    return res;
  } else {
    throw gim_exception(GIM_ERR_UNKNOWN);
  } /*if*/
}

void gim_connection::devinfo(char* info, size_t info_size) throw (struct gim_exception)
{
  if (fd != -1) {
    int res = gim_client_devinfo(fd, info, info_size);
    if (res < 0) {
      throw gim_exception(res);
    } /*if*/
  } else {
    throw gim_exception(GIM_ERR_UNKNOWN);
  } /*if*/
}

int gim_connection::servinfo_len() throw (struct gim_exception)
{
  if (fd != -1) {
    int res = gim_client_servinfo(fd, 0, 0);
    if (res < 0) {
      throw gim_exception(res);
    } /*if*/
    return res;
  } else {
    throw gim_exception(GIM_ERR_UNKNOWN);
  } /*if*/
}

void gim_connection::servinfo(char* info, size_t info_size) throw (struct gim_exception)
{
  if (fd != -1) {
    int res = gim_client_servinfo(fd, info, info_size);
    if (res < 0) {
      throw gim_exception(res);
    } /*if*/
  } else {
    throw gim_exception(GIM_ERR_UNKNOWN);
  } /*if*/
}

void gim_connection::disable_paste() throw (struct gim_exception)
{
  if (fd != -1) {
    int res = gim_client_disable_paste(fd);
    if (res != GIM_DISABLE_PASTE_SUCCESS) {
      throw gim_exception(res);
    } /*if*/
  } /*if*/
}

int gim_connection::has_event() throw (struct gim_exception)
{
  if (fd == -1) {
    return 0;
  } /*if*/
  fd_set readset;
  FD_ZERO(&readset);
  FD_SET(fd, &readset);
  struct timeval tv;
  tv.tv_sec = tv.tv_usec = 0;
  int res = select(fd+1, &readset, &readset, &readset, &tv);
  if (res < 0) {
    throw gim_exception(GIM_ERR_UNIX);
  } /*if*/
  return (res > 0);
}

void gim_connection::get_event(struct gim_event_t* p_ev) throw (struct gim_exception)
{
  if (fd != -1) {
    int res = gim_get_event(fd, p_ev);
    if (res != GIM_GET_EVENT_SUCCESS) {
      throw gim_exception(res);
    } /*if*/
  } else {
    throw gim_exception(GIM_ERR_UNKNOWN);
  } /*if*/
}

#endif // __cplusplus
